import time
import http.client as httplib
from datetime import datetime, timezone

# %%
def _now():
    return datetime.utcnow().replace(tzinfo=timezone.utc)

# %%
def _internet(url='www.google.com', return_status=False, verbose=3):
    # Check whether server is still alive.
    counter=1
    status=False
    sleepinsec=60

    while status is False:
        conn = httplib.HTTPConnection(url, timeout=5)
        try:
            conn.request("HEAD", "/")
            conn.close()
            status=True
        except:
            print('[slacki] >ERROR: [%s] No internet connection? Trying again in 60 sec.. [attempt %s]' %(_now().strftime('%d-%m-%Y %H:%M'), counter))
            status=False
            time.sleep(sleepinsec)
            counter=counter+1

    if counter>1:
        if verbose>=3: print('[slacki] >[%s] Internet connection re-established after after %s attempts.' % (_now().strftime('%d-%m-%Y %H:%M'), counter))
    
    if return_status:
        return(status)


#%% Check the connection with slack. Do this only once otherwise it will result in to many requests error
def _slack(sc, return_status=False):
    # Check connection
    counter=1
    status=False
    sleepinsec=60

    while status is False:
        try:
            getconnection=sc.rtm.connect()
            status = getconnection.successful
        except:
            print('[slacki] >ERROR: [%s] No Slack (aka internet) connection? Trying again in 60 sec.. [attempt %s]' %(_now().strftime('%d-%m-%Y %H:%M'), counter))
            status=False
            time.sleep(sleepinsec)
            counter=counter+1

    if counter>1:
        if verbose>=1: print('[slacki] >[%s] Slack connection re-established after after %s attempts.' % (_now().strftime('%d-%m-%Y %H:%M'), counter))

    if return_status:
        return(status)



#--------------------------------------------------------------------------
# Name        : slacki.py
# Author      : E.Taskesen
# Contact     : erdogant@gmail.com
#--------------------------------------------------------------------------

# Libraries
import slacki.utils.check_connection as check_connection
from datetime import datetime
from slacker import Slacker
import traceback
import numpy as np
import time

# %%
class slacki():
    """Python package slacki for reading and posting in slack groups."""

    def __init__(self, channel=None, token=None, response_time=60, verbose=3):
        """Initialize slacki with user-defined parameters."""
        if token is None: raise Exception('Token should be a valid id.')
        # Check internet connection
        check_connection._internet()
        # Check channel name
        channel = get_channel(channel)
        try:
            sc = Slacker(token)
            # check_connection._slack(sc)
        except Exception as e:
            if verbose>1: print(('[slacki] >ERROR: Could not import token: [%s]' % (str(e))))

        # Store in object
        self.sc = sc
        self.channel = channel
        self.channel_id = sc.channels.get_channel_id(channel[1:])
        self.token = token
        self.response_time = response_time
        self.verbose = verbose
        # Make channel if not exists
        make_channel(self.sc, self.channel, verbose=verbose)

    # Retrieve posts
    def retrieve_posts(self, date_from=None, date_to=None, n=None, verbose=3):
        """Retrieve posts

        Parameters
        ----------
        date_from : str: "%Y-%m-%d %H:%M:%S"
            Date From
        date_to : str: "%Y-%m-%d %H:%M:%S"
            Date To
        n : int, default: None
            Retrieve the n latest posts. If set to default=None, all is retrieved.
        verbose : int, optional
            Verbosity. The default is 3.

        Returns
        -------
        list of dict containing posts.

        """
        # Check internet connection
        check_connection._internet()
        oldest = None
        latest = None
        get_posts = []

        # Set date
        if date_from is not None:
            oldest = (datetime.strptime(date_from, "%Y-%m-%d %H:%M:%S") - datetime(1970, 1, 1)).total_seconds()
        if date_to is not None:
            latest = (datetime.strptime(date_to, "%Y-%m-%d %H:%M:%S") - datetime(1970, 1, 1)).total_seconds()

        # Retrieve channel IDs of not present yet
        if self.channel_id is None:
            self.channel_id = self.sc.channels.get_channel_id(self.channel[1:])

        # Try to retrieve posts
        try:
            if self.channel_id is None:
                if verbose>=3: print('[slacki] Could not retrieve ID for channel %s. If the channel is set on private, change it to normal and try again.' % (self.channel))
            else:
                # Retrieve History
                GEThistory = self.sc.channels.history(self.channel_id, oldest=oldest, latest=latest, count=n)
                get_posts = GEThistory.body['messages']
        except:
            if verbose>=1: print(traceback.print_exc())

        return(get_posts)

    # Snooze
    def snooze(self, minutes=None, return_status=False, verbose=3):
        """Snooze the slack messages for certain time.

        Parameters
        ----------
        minutes : int, default: None
            Number of minutes to snooze.
        return_status : bool, default: False
            Return the status of the snoozing action.
        verbose : int, optional
            Verbosity. The default is 3.

        Returns
        -------
        status : bool

        """
        try:
            if minutes is not None:
                self.sc.dnd.set_snooze(num_minutes=minutes)        
                if verbose>=3: print('[slacki] >Snoozer for %s is set on %.d minutes' %(self.channel, minutes))
                OK = True
        except Exception as e:
            if verbose>=1: print(('[slacki] >ERROR: [%s]' % (str(e))))
            OK = False

        # Return
        if return_status:
            return(OK)

    # Post message in channel
    def post(self, queries, icon_url=None, return_status=False, verbose=3):
        """Post messages on slack.

        Parameters
        ----------
        queries : list
            list with strings containing messages to be posted.
        icon_url : str
            String with icon url.
        return_status : bool, default: False
            Return the status of the snoozing action.
        verbose : int, optional
            Verbosity. The default is 3.

        Returns
        -------
        status : bool

        """
        if queries is None: raise Exception('Queries should be a string or list with strings.')
        # Check internet connection
        check_connection._internet()
        # Make list if required
        if isinstance(queries, str):
            queries=[queries]
        # Post messages
        try:
            for query in queries:
                self.sc.chat.post_message(self.channel, text=query, username=None, icon_url=None)
                if verbose>=5: print('[slacki] [%s] is posted' %(query))
            postOK = True
        except Exception as e:
            if verbose>1: print(('[slacki] >ERROR: [%s]' % (str(e))))
            postOK = False
        # Return
        if return_status:
            return(postOK)

    # Post file in channel
    def post_file(self, file, query='File upload message', title=None, return_status=False, verbose=3):
        """Post file in slack message.

        Parameters
        ----------
        file : str
            Pathname to the file to be posted in slack.
        query : str
            string containing messages to be posted with the file.
        title : str
            Title of the file to be posted with the file.
        return_status : bool, default: False
            Return the status of the snoozing action.
        verbose : int, optional
            Verbosity. The default is 3.

        Returns
        -------
        status : bool

        """
        # Check internet connection
        check_connection._internet()
        try:
            self.sc.files.upload(file_=file, channels=self.channel, title=title, initial_comment=query)
            if verbose>=5: print('[slacki] File is posted')
            postOK = True
            # print(obj.successful, obj.__dict__['body']['channel']['id'])
        except Exception as e:
            if verbose>=1: print(('[slacki] >ERROR: [%s]' % (str(e))))
            postOK = False
        # Return
        if return_status:
            return(postOK)

    # Post message in channel
    def info(self, verbose=3):
        """Retrieve user information for the slack-group

        Parameters
        ----------
        verbose : int, optional
            Verbosity. The default is 3.

        Returns
        -------
        list with users.

        """
        # Check internet connection
        check_connection._internet()
        users = dict()
        try:
            # Find all users
            getusers = self.sc.users.list().body
            users['realname'] = list(map(lambda x:x['real_name'], getusers['members']))
            users['name'] = list(map(lambda x:x['name'], getusers['members']))
            for u in getusers['members']:
                # print(f'User: {u["name"]}, Real Name: {u["real_name"]}, Time Zone: {u["tz_label"]}.')
                if verbose>=3: print('User: %s, Real Name: %s, Time Zone: %s.' % (u['name'], u['real_name'], u['tz_label']))
                # print(f'Current Status: {u["profile"]["status_text"]}')
                # Get image data and show
                # Image(user['profile']['image_192'])
            users['info']=getusers
        except Exception as ex:
            if verbose>=1: print(('[slacki] >ERROR: [%s]' % (str(ex))))
        return(users)


    # %%
    def listen(self, date_from=None, date_to=None, n=None, response_time=None, verbose=3):
        if response_time is None: response_time = self.response_time
        get_posts = self.retrieve_posts(date_from=date_from, date_to=date_to, n=n, verbose=verbose)
            
        # Setup questions
        tasks=dict()
        tasks['figure']=False
        tasks['summary']=False
        tasks['status']=False
        tasks['advice']=False
        tasks['last_trade']=False
        
        # Run over all posts
        for post in get_posts:
            if post.get('client_msg_id')!=None:# and post.get('type') == 'message' and not 'subtype' in post.get('type'): # Make sure its not a bot, otherwise it starts singning.
                if '@figure' in post['text']:
                    tasks['figure']=True
                elif '@summary' in post['text']:
                    tasks['summary']=True
                elif '@status' in post['text']:
                    tasks['status']=True
                elif '@advice' in post['text']:
                    tasks['advice']=True
                elif '@last_trade' in post['text']:
                    tasks['last_trade']=True
                elif '@help' in post['text']:
                    if verbose>=3: print('[BOT] My response time is within %d seconds. I listen to: @figure, @summary, @status, @advice, @last_trade, @help' % (response_time))
                else:
                    if verbose>=3: print('[BOT] I can not do <%s>' % (post['text']))

        # Retrieve data or return
        if not np.any([*tasks.values()]):
            return
        else:
            print('Do some stuff')
            return

# %% Create channel
def make_channel(sc, channel, verbose=3):
    """Create channel."""
    try:
        sc.channels.create(channel[1:])
        if verbose>=3: print('[slacki] >Channel [%s] is succesfull created' % (channel))
    except Exception as e:
        if verbose>=3: print('[slacki] >Channel [%s] already exists.' % (channel))


# %% Post message in channel
def get_channel(channel):
    if (channel is None) or (channel=='') or (len(channel)<3): raise Exception('Channel should be a valid name; can not be None or "" or less then 3 chars.')
    if channel[0]!='#':
        channel = '#' + channel
    
    return(channel)



# Setup your slack account with a token:
# Make token as following:
#   1. https://api.slack.com/custom-integrations/legacy-tokens
#   2. Scroll down a bit untill you see 'create token'
#   3. Click on 'create token'

# %%
import slacki as slacki
print(dir(slacki))
print(slacki.__version__)

# %%
from slacki import slacki
token='xoxp-123234234235-123234234235-123234234235-adedce74748c3844747aed48499bb'

# %%
sc = slacki(channel='new_channel', token=token)

# Get some info about the users
users = sc.info()

 # Send messages
queries=['message 1','message 2']
sc.post(queries)

# Snoozing
sc.snooze(minutes=1)

# Post file
sc.post_file(file='./data/slack.png', title='Nu ook met figuren uploaden :)')

# listen (retrieve only last message)
out = sc.retrieve_posts(n=3)

# %% listen (retrieve only last message)
sc.listen()



import setuptools
import re

# versioning ------------
VERSIONFILE="slacki/__init__.py"
getversion = re.search( r"^__version__ = ['\"]([^'\"]*)['\"]", open(VERSIONFILE, "rt").read(), re.M)
if getversion:
    new_version = getversion.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

# Setup ------------
with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
     install_requires=['slacker','numpy'],
     python_requires='>=3',
     name='slacki',
     version=new_version,
     author="Erdogan Taskesen",
     author_email="erdogant@gmail.com",
     description="Python package slacki for reading and posting in slack groups.",
     long_description=long_description,
     long_description_content_type="text/markdown",
     url="https://github.com/erdogant/slacki",
	 download_url = 'https://github.com/erdogant/slacki/archive/'+new_version+'.tar.gz',
     packages=setuptools.find_packages(), # Searches throughout all dirs for files to include
     include_package_data=True, # Must be true to include files depicted in MANIFEST.in
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
 )



from slacki.slacki import slacki

__author__ = 'Erdogan Tasksen'
__email__ = 'erdogant@gmail.com'
__version__ = '0.1.2'

# module level doc-string
__doc__ = """
slacki
=====================================================================

Description
-----------
Python package slacki for reading and posting in slack groups.

Example
-------
>>> from slacki import slacki
>>> sc = slacki(channel='new_channel', token='xoxp-123234234235-123234234235-123234234235-adedce74748c3844747aed48499bb')
>>> 
>>> # Get some info about the users
>>> users = sc.info()
>>> 
>>> # Send messages
>>> queries=['message 1','message 2']
>>> sc.post(queries)
>>> 
>>> # Snoozing
>>> sc.snooze(minutes=1)
>>> 
>>> # Post file
>>> sc.post_file(file='./data/slack.png', title='Nu ook met figuren uploaden :)')
>>> 
>>> # listen (retrieve only last message)
>>> out = sc.retrieve_posts(n=3)

References
----------
https://github.com/erdogant/slacki

"""
